import h5py

def setOutputTimesteps(algorithm, timesteps):
    """helper routine to set timestep information"""
    executive= algorithm.GetExecutive()
    outInfo = executive.GetOutputInformation(0)

    outInfo.Remove(executive.TIME_STEPS())
    for timestep in timesteps:
        outInfo.Append(executive.TIME_STEPS(), timestep)

    outInfo.Remove(executive.TIME_RANGE())
    outInfo.Append(executive.TIME_RANGE(), timesteps[0])
    outInfo.Append(executive.TIME_RANGE(), timesteps[-1])

def getFileList(fpath, fprefix, fpostfix):
    """get all files"""
    import os

    files=[]
    for i in os.listdir(fpath):
        if os.path.isfile(os.path.join(fpath,i)) \
           and fprefix in i \
           and fpostfix in i:
            files.append(i)
    files.sort()
    
    return files

def getTimeTuple(fname, fprefix, fpostfix):
    """get timetuple from file name"""
    
    import time
    import datetime

    ftime_str  = fname[len(fprefix):-len(fpostfix)]
    ftime_str  = fname[0:10]
    print ftime_str
    #ftimetuple = time.mktime(datetime.datetime.strptime(ftime_str,"%Y%m%d_%H%M").timetuple())
    ftimeseconds = time.mktime(datetime.datetime.strptime(ftime_str,"%Y%m%d%H").timetuple())
    print ftimeseconds

    return ftimeseconds

def getTimestepsFromFiles(files, fprefix, fpostfix):
    """generate timestap from filename"""

    ref_ftime = 0.0
    ftime_sec = []
    for i, fname in enumerate(files):
        cur_ftime = getTimeTuple(fname, fprefix, fpostfix)
        if i == 0:
            ref_ftime = cur_ftime
        ftime_sec.append(cur_ftime -ref_ftime)
        
    ftime_h = [ x / 3600 for x in ftime_sec ]  
    ftime_h_int = [int(i) for i in ftime_h]
    
    return ftime_h_int
    
# get all files
# fpath="/homeb/zam/zilken/JURECA/projekte/o.stein/data/"

fpath = "/data/slmet/slmet111/met_data/ecmwf/era5/netcdf4/2017/"
fprefix="201709"
fpostfix="_ml.nc"
files = getFileList(fpath, fprefix, fpostfix)
print files
print len(files)
#quit()


timestamps = getTimestepsFromFiles(files, fprefix, fpostfix)
print timestamps

scalars=["cc", "ciwc", "clwc", "q", "d", "vo", "o3", "w"]
fileIn = files[0]
coordFile = "./coordinates.h5"
outfilename = "./structuredgrid_201709.xdmf"
# open hdf5 file
fIn = h5py.File(fpath+fileIn, "r")
# read attributes to get size of data
latIn = fIn['/lat'] # open one attribute to get the sizes
lonIn = fIn['/lon'] # open one attribute to get the sizes
levIn = fIn['/lev_2'] # open one attribute to get the sizes
#levIn = fIn['/lev'] # open one attribute to get the sizes
num_x = levIn.shape[0]
num_y = latIn.shape[0]
num_z = lonIn.shape[0]
fIn.close()

f = open(outfilename, 'w')

f.write( "<?xml version =\"1.0\" ?>\n")
f.write( "<!DOCTYPE xdmf SYSTEM \"Xdmf.dtd\" []>\n")
f.write( "<Xdmf Version=\"2.0\">\n")
f.write( "<Domain>\n")
f.write( "<Grid Name=\"Main Grid\" GridType=\"Collection\" CollectionType=\"Temporal\">\n")
f.write( "\n")

f.write( "<Time TimeType=\"List\">\n")
f.write( "   <DataItem Dimensions=\"%d\" Format=\"XML\" NumberType=\"Float\" Precision=\"4\">\n" % (len(files)))
for i in range(len(files)):
  f.write("%d " % (timestamps[i]))
f.write("\n")
f.write(" </DataItem>\n")
f.write("</Time>\n")

for i in range(len(files)):
  f.write( "<Grid Name=\"Timestep_%03d\" GridType=\"Uniform\">\n" % (i))
  f.write( "   <Topology TopologyType=\"3DSMesh\" NumberOfElements=\"%d %d %d\"/>\n" % (num_x, num_y, num_z))
  f.write( "   <Geometry GeometryType=\"X_Y_Z\">\n")
  f.write( "      <DataItem Dimensions=\"%d %d %d\" NumberType=\"Float\" Precision=\"4\" Format=\"HDF\">\n" % (num_x, num_y, num_z))
  f.write( "        " + coordFile +":/x\n")
  f.write( "      </DataItem>\n")
  f.write( "      <DataItem Dimensions=\"%d %d %d\" NumberType=\"Float\" Precision=\"4\" Format=\"HDF\">\n" % (num_x, num_y, num_z))
  f.write( "        " + coordFile +":/y\n")
  f.write( "      </DataItem>\n")
  f.write( "      <DataItem Dimensions=\"%d %d %d\" NumberType=\"Float\" Precision=\"4\" Format=\"HDF\">\n" % (num_x, num_y, num_z))
  f.write( "        " + coordFile +":/z\n")
  f.write( "      </DataItem>\n")
  f.write( "   </Geometry>\n")

  for j in range(len(scalars)):
      f.write( "   <Attribute Name=\"" + scalars[j]+ "\" AttributeType=\"Scalar\" Center=\"Node\">\n")
      f.write( "      <DataItem ItemType=\"Hyperslab\" Dimensions=\"%d %d %d\" NumberType=\"Float\" Precision=\"4\" Format=\"HDF\">\n" % (num_x, num_y, num_z))
      f.write( "         <DataItem Dimensions=\"3 4\" Format=\"XML\">\n")
      f.write( "            0 0 0 0\n" )
      f.write( "            1 1 1 1\n")
      f.write( "            1 %d %d %d\n" % (num_x, num_y, num_z))
      f.write( "         </DataItem>\n")
      f.write( "         <DataItem Dimensions=\"%d %d %d %d\" NumberType=\"Float\" Precision=\"4\" Format=\"HDF\">\n" % (1, num_x, num_y, num_z))
      f.write( "               " + fpath + files[i] + ":/" + scalars[j] + "\n")
      f.write( "         </DataItem>\n")
      f.write( "      </DataItem>\n")
      f.write( "   </Attribute>\n")
  f.write("</Grid>\n")

f.write("\n")
f.write("</Grid>\n")
f.write("</Domain>\n")
f.write("</Xdmf>\n")
f.close()
